#property script_show_inputs

input double inTP = 0; //     
input double inSL = 0; //     

#include <fxsaber\SingleTesterCache\SingleTesterCache.mqh> // https://www.mql5.com/ru/code/27611
#include <fxsaber\MultiTester\MTTester.mqh> // https://www.mql5.com/ru/code/26132

double GetProfitMFE( const double MFE, const TesterPositionProfit &Positions[] )
{
  double Res = 0;

  for (int i = ArraySize(Positions) - 1; i >= 0; i--)
    Res += (Positions[i].mfe >= MFE) ? MFE : Positions[i].profit;

  return(Res);
}

double GetProfitMAE( const double MAE, const TesterPositionProfit &Positions[] )
{
  double Res = 0;

  for (int i = ArraySize(Positions) - 1; i >= 0; i--)
    Res += (Positions[i].mae <= MAE) ? MAE : Positions[i].profit;

  return(Res);
}

double GetMaxProfitMFE( const TesterPositionProfit &Positions[] )
{
  const int Size = ArraySize(Positions);

  double ProfitMax = -DBL_MAX;

  for (int i = ArraySize(Positions) - 1; i >= 0; i--)
  {
    const double Profit = GetProfitMFE(Positions[i].mfe, Positions);

    if (Profit > ProfitMax)
      ProfitMax = Profit;
  }

  return(ProfitMax);
}

double GetMaxProfitMAE( const TesterPositionProfit &Positions[] )
{
  const int Size = ArraySize(Positions);

  double ProfitMax = -DBL_MAX;

  for (int i = ArraySize(Positions) - 1; i >= 0; i--)
  {
    const double Profit = GetProfitMAE(Positions[i].mae, Positions);

    if (Profit > ProfitMax)
      ProfitMax = Profit;
  }

  return(ProfitMax);
}

string ToString( const double Value1, const double Value2 )
{
  return((string)Value1 + " (" + DoubleToString(100 * Value1 / Value2, 2) + "%)");
}

void OnStart()
{
  uchar Bytes2[];

  if (MTTESTER::GetLastTstCache(Bytes2) != -1) //     -  
  {
    const SINGLETESTERCACHE SingleTesterCache(Bytes2);

    const double Profit = SingleTesterCache.Summary.TesterStatistics(STAT_PROFIT);

    const double ProfitTP = (inTP > 0) ? GetProfitMFE(inTP, SingleTesterCache.Positions) : 0;
    const double ProfitSL = (inSL < 0) ? GetProfitMFE(inSL, SingleTesterCache.Positions) : 0;

    const double MaxProfitMFE = GetMaxProfitMFE(SingleTesterCache.Positions);
    const double MaxProfitMAE = GetMaxProfitMAE(SingleTesterCache.Positions);

  #define TOSTRING(A) #A + " = " + ToString(A, Profit) + "\n"

    const string Str = TOSTRING(Profit) + TOSTRING(MaxProfitMFE) + TOSTRING(MaxProfitMAE) +
                       ((inTP > 0) ? "\n    " + (string)inTP + ",  " + TOSTRING(ProfitTP) : NULL) +
                       ((inSL < 0) ? "\n    " + (string)inSL + ",  " + TOSTRING(ProfitSL) : NULL);
    MessageBox(Str);
  }
}